.include "1200def.inc"

; /------------------------------------\
; |      MIDI-to-Sync24 converter      |
; |	 firmware version 1.0-AVR      |
; |	    By Bojan Burkeljc	       |
; |====================================|
; | 	Microprocessor: AT90S1200      |
; |            Cristal:  4 MHz         |
; |====================================|
; | Started coding on: 2nd June 2002   |
; | Ended coding on: 19th August 2002  |
; \------------------------------------/

.def	temp		=R29
.def	midi_temp	=R28
.def	bit_cnt		=R27
.def	int_temp	=R26
.def	midi_temp	=R25
.def	midi_bit_cnt	=R24
.def	count		=R23
.def	count_lo	=R22
.def	sync_out	=R21
.def	sync_out_pre	=R20
.def	state_48	=R19
.def	scale		=R18
.def	active_cnt0	=R2
.def	active_cnt1	=R17
.def	temp_status	=R0

; Port D pins
.set	SYNC_ON		=0
.set	SYNC_CONT	=1
.set	MIDI_IN		=2
.set	SYNC_24		=3
.set	MIDI_ACTIVE	=4
.set	MIDI_THRU	=5
.set	SYNC_48		=6


; Constant definition
.equ	TMR_START_CONST	  =-0x7F
.equ	TMR_DATA_CONST	  =0x7D
.equ	PULSE_LENGHT	  =63
.equ	MIDI_ACTIVE_CONST =15
;
;
;
;
.ORG	0x000
	rjmp	INIT
.ORG	0x001
	rjmp	MIDI_START_BIT
.ORG	0x002
	rjmp	MIDI_DATA_BIT
.ORG	0x003
	rjmp	PC+1
.ORG	0x004
INIT:
	clr	temp
	out	PORTB,temp
	ldi	temp,0b00110000
	out	PORTD,temp
	ldi	temp,0xFF
	out	DDRB,temp
	ldi	temp,0b11111011
	out	DDRD,temp
	ldi	temp,0b10000000		; Disable analog comparators
	out	ACSR,temp
	ldi	temp,0b00000001		; Initialize TMR without prescaler
	out	TCCR0,temp
	ldi	scale,3
	ldi	state_48,1
	clr	count
	clr	sync_out
	clr	sync_out_pre
;
; -=[ Turn On synchronization procedure ]=-	OK OK OK
;
; This procedure makes sure that MIDI is not trasmiting any byte (which 
; might cause MIDI byte misreading) when we turn on this device
;
TURN_ON:
	ldi	count,9
TURN_ON_CHECK:
	sbis	PIND,MIDI_IN		; 2 1
	rjmp	TURN_ON_REINIT		; - 2 (3)
	dec	count			; 1
	breq	IRQ_INIT		; 1 2 ()
	rjmp	PC+1			; 2
SYNCHRONIZE_AGAIN:			; (6)
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rjmp	TURN_ON_CHECK		; 2 (128)
TURN_ON_REINIT:				; (3)
	ldi	count,9			; 1
	rjmp	SYNCHRONIZE_AGAIN	; 2 (6)
;	
IRQ_INIT:
	ldi	temp,1<<ISC01		; INT0 interrupt edge-triggered
	out	MCUCR,temp
	ldi	temp,1<<INT0
	out	GIMSK,temp		; INT0 interrupt enable
	clr	temp
	out	TIMSK,temp		; Disable TMR interrupt
	sei				; Global interrupt enable
;
; -=[ Main Loop ]=-
;
MAIN_LOOP:
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rjmp	PC+1			; 2
COUNTDOWN:				; (102)
	sbic	PIND,SYNC_48		; 2 1
	dec	count			; - 1
	sbis	PIND,SYNC_48		; 2 1
	dec	count_lo		; - 1
	brne	COUNT_NOT_ZERO		; 1 2 (108)
	clr	temp			; 1
	out	PORTB,temp		; 1
	cbi	PORTD,SYNC_CONT		; 2
	cbi	PORTD,SYNC_24		; 2
	dec	state_48		; 1
	brne	STATE48_NOT_ZERO	; 1 2 (116)
	cbi	PORTD,SYNC_48		; 2
	inc	state_48		; 1
	rjmp	ACTIVE_CNT		; 2 (120)


COUNT_NOT_ZERO:				; (108)
	sbis	PIND,SYNC_48		; 2 1
	ldi	count,PULSE_LENGHT	; - 1
	sbic	PIND,SYNC_48		; 2 1
	ldi	count_lo,PULSE_LENGHT	; - 1
	rjmp	PC+1			; 2
	rjmp	PC+1			; 2
	rjmp	PC+1			; 2
	rjmp	ACTIVE_CNT		; 2 (120)
;
STATE48_NOT_ZERO:			; (116)	
	in	temp,PIND		; 1
	bst	state_48,0		; 1
	bld	temp,SYNC_48		; 1
	out	PORTD,temp		; 1 (120)

ACTIVE_CNT:				; (120)
	dec	active_cnt0		; 1
	brne	ACTIVE_CNT_NOT_ZERO	; 1 2
	dec	active_cnt1		; 1 
	brne	ACTIVE_CNT_NOT_ZERO1	; 1 2
	sbi	PORTD,MIDI_ACTIVE	; 2
	rjmp	MAIN_LOOP		; 2 (128)

ACTIVE_CNT_NOT_ZERO:			;(~3)
	rjmp	PC+1			; 2
ACTIVE_CNT_NOT_ZERO1:			;(~5)
	nop				; 1
	rjmp	MAIN_LOOP		; 2 (128)
;
;
DELAY_20:
	rjmp	PC+1	; 2
	rjmp	PC+1	; 2
DELAY_16:
	rjmp	PC+1	; 2
DELAY_14:
	rjmp	PC+1	; 2
DELAY_12:
	rjmp	PC+1	; 2
DELAY_10:		; 3
	rjmp	PC+1	; 2
	nop		; 1
	ret		; 4



;
; Interrupt routines
;
MIDI_START_BIT:
	in	temp_status,SREG
;
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	rcall	DELAY_20		; 20	
	cbi	PORTD,MIDI_THRU		; MIDI Thru update
;
	clr	int_temp
	out	GIMSK,int_temp		; Disable INT0 interrupt 
	ldi	int_temp,1<<TOIE0
	out	TIMSK,int_temp		; Set TMR interrupt enable
	out	TIFR,int_temp		; Enable TMR overflow
	clr	int_temp		; ...
	out	MCUCR,int_temp		; ...

	ldi	midi_bit_cnt,9		; Set MIDI bit counter
	clr	active_cnt0
	ldi	active_cnt1,MIDI_ACTIVE_CONST
	cbi	PORTD,MIDI_ACTIVE

	ldi	int_temp,TMR_START_CONST
	out	TCNT0,int_temp		; Set TMR0 value
	out	SREG,temp_status
	reti
;
MIDI_DATA_BIT:
	in	temp_status,SREG
;
	in	int_temp,PIND
	bst	int_temp,MIDI_IN	
	bld	int_temp,MIDI_THRU
	out	PORTD,int_temp
;
	dec	midi_bit_cnt
	breq	MIDI_BYTE_END
	ror	midi_temp
	bld	midi_temp,7

	in	int_temp,TCNT0
	subi	int_temp,TMR_DATA_CONST
	out	TCNT0,int_temp
	out	SREG,temp_status
	reti
;
MIDI_BYTE_END:
	subi	midi_temp,0xF8
	breq	MIDI_CLOCK_M
	subi	midi_temp,2
	breq	MIDI_START_M
	dec	midi_temp
	breq	MIDI_CONT_M
	dec	midi_temp
	breq	MIDI_STOP_M
	rjmp	REINIT_START_BIT

;
MIDI_CLOCK_M:
	ldi	count,PULSE_LENGHT
	dec	scale
	brne	SCALE_NOT
	sbis	PIND,SYNC_ON
	rjmp	SCALE_NOT
	dec	sync_out
	and	sync_out_pre,sync_out
	sec
	rol	sync_out_pre
	out	PORTB,sync_out_pre
	ldi	scale,3
	sbi	PORTD,SYNC_24
	sbi	PORTD,SYNC_48
	ldi	state_48,3
	rjmp	REINIT_START_BIT
;
SCALE_NOT:
	mov	sync_out_pre,sync_out
	com	sync_out_pre
	sbi	PORTD,SYNC_24
	sbi	PORTD,SYNC_48
	ldi	state_48,3
	rjmp	REINIT_START_BIT
;
MIDI_START_M:
	sbi	PORTD,SYNC_ON
	ldi	scale,1
	clr	sync_out
	ser	sync_out_pre
	rjmp	REINIT_START_BIT
;
MIDI_CONT_M:
	sbi	PORTD,SYNC_CONT
	ldi	scale,1
	clr	sync_out
	ser	sync_out_pre
	sbi	PORTD,SYNC_ON
	rjmp	REINIT_START_BIT
;
MIDI_STOP_M:
	cbi	PORTD,SYNC_ON
	rjmp	REINIT_START_BIT

;
REINIT_START_BIT:
	ldi	int_temp,1<<ISC01	; ...
	out	MCUCR,int_temp		; ...
	ldi	int_temp,1<<INT0	;
	out	GIMSK,int_temp		; INT0 interrupt enable
	clr	int_temp		;
	out	TIMSK,int_temp		; Disable TMR interrupt
	out	SREG,temp_status
	reti

	